/*
 * Copyright (c) 2015 Nathan Osman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#ifndef QHTTPENGINE_QSOCKETPAIR_H
#define QHTTPENGINE_QSOCKETPAIR_H

#include <QObject>
#include <QTcpServer>
#include <QTcpSocket>

/**
 * @brief Create a pair of connected TCP sockets
 *
 * This class simplifies the process of creating two TCP sockets that are
 * connected to each other. Once isConnected() returns true, data can be
 * written and read from each socket.
 */
class QSocketPair : public QObject
{
    Q_OBJECT

public:

    QSocketPair();

    bool isConnected() const {
        return mClientSocket.isValid() && mServerSocket && mServerSocket->isValid();
    }

    QTcpSocket *client() {
        return &mClientSocket;
    }

    QTcpSocket *server() {
        return mServerSocket;
    }

private Q_SLOTS:

    void onNewConnection();

private:

    QTcpServer mServer;

    QTcpSocket mClientSocket;
    QTcpSocket *mServerSocket;
};

#endif // QHTTPENGINE_QSOCKETPAIR_H
