/*
 * Copyright (c) 2015 Nathan Osman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <QString>

#include "QHttpEngine/qibytearray.h"

QIByteArray::QIByteArray()
{}

QIByteArray::QIByteArray(const QByteArray &other)
    : QByteArray(other)
{}

QIByteArray::QIByteArray(const char *data, int size)
    : QByteArray(data, size)
{}

bool operator==(const QIByteArray &a1, const QIByteArray &a2)
{
    return a1.toLower() == a2.toLower();
}

bool operator==(const QIByteArray &a1, const QString &a2)
{
    return a1.toLower() == a2.toLower();
}

bool operator==(const QString &a1, const QIByteArray &a2)
{
    return a2 == a1;
}

bool operator==(const QIByteArray &a1, const QByteArray &a2)
{
    return a1.toLower() == a2.toLower();
}

bool operator==(const QByteArray &a1, const QIByteArray &a2)
{
    return a2 == a1;
}

bool operator==(const QIByteArray &a1, const char *a2)
{
    return a1.toLower() == QByteArray(a2).toLower();
}

bool operator==(const char *a1, const QIByteArray &a2)
{
    return a2 == a1;
}
